# Generated by roxygen2: do not edit by hand

S3method(coef,barma)
S3method(fitted,barma)
S3method(forecast,barma)
S3method(plot,barma)
S3method(print,barma)
S3method(print,summary.barma)
S3method(residuals,barma)
S3method(summary,barma)
export(barma)
export(fim_barma)
export(loglik_barma)
export(make_link_structure)
export(score_vector_barma)
export(simu_barma)
export(start_values)
importFrom(forecast,forecast)
importFrom(forecast,ggAcf)
importFrom(forecast,ggPacf)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_text)
importFrom(ggplot2,geom_density)
importFrom(ggplot2,geom_histogram)
importFrom(ggplot2,geom_hline)
importFrom(ggplot2,geom_line)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_colour_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,stat_qq)
importFrom(ggplot2,stat_qq_line)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_minimal)
importFrom(grid,grid.draw)
importFrom(gridExtra,arrangeGrob)
importFrom(rlang,.data)
importFrom(stats,acf)
importFrom(stats,coef)
importFrom(stats,dbeta)
importFrom(stats,density)
importFrom(stats,fitted)
importFrom(stats,frequency)
importFrom(stats,is.ts)
importFrom(stats,lm.fit)
importFrom(stats,make.link)
importFrom(stats,optim)
importFrom(stats,pacf)
importFrom(stats,pbeta)
importFrom(stats,pchisq)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rbeta)
importFrom(stats,residuals)
importFrom(stats,start)
importFrom(stats,time)
importFrom(stats,ts)
importFrom(stats,var)
