% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/print.barma.R
\name{print.barma}
\alias{print.barma}
\title{Print Method for a barma Model}
\usage{
\method{print}{barma}(x, ...)
}
\arguments{
\item{x}{A fitted model object of class `"barma"`.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
Invisibly returns the original object `x`.
}
\description{
S3 method for printing a summary of a fitted `"barma"` model object.
}
\details{
This function provides a concise summary, showing the function call 
that generated the model, the link function used, and the final 
estimated coefficients.
}
