% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fitted.barma.R
\name{fitted.barma}
\alias{fitted.barma}
\title{Extract Fitted Values from a barma Model}
\usage{
\method{fitted}{barma}(object, ...)
}
\arguments{
\item{object}{A fitted model object of class `"barma"`.}

\item{...}{Additional arguments (currently ignored).}
}
\value{
A `ts` object of the fitted mean values.
}
\description{
S3 method for extracting the fitted mean values (mu-hat) from a fitted 
model object of class `"barma"`.
}
\details{
The fitted values are returned as a time series (`ts`) object, 
matching the time properties of the original input `y`. The first
`max_lag` observations are `NA`, as they cannot be fitted.
}
